//+------------------------------------------------------------------+
//|                                                     Blau_HLM.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp." //  -
#property link      "http://www.mql5.com"                       //    -
#property description "q-period High-Low Momentum (William Blau)" //   mql5-
#include <WilliamBlau.mqh>              //   (   )
//---  
#property indicator_separate_window     //     
#property indicator_buffers 6           //     
#property indicator_plots   1           //     
//---   #0 (Main)
#property indicator_label1  "HLM"       //     #0
#property indicator_type1   DRAW_LINE   //  : DRAW_LINE - 
#property indicator_color1  Blue        //    : Blue - 
#property indicator_style1  STYLE_SOLID //  : STYLE_SOLID -  
#property indicator_width1  1           //  
//---  
input int    q=2;  // q - ,    
input int    r=20; // r -  1- EMA,   
input int    s=5;  // s -  2- EMA,     
input int    u=3;  // u -  3- EMA,     
//---     
double MainBuffer[];     // u- 3- EMA (   #0)
double HMUBuffer[];      // q-  -
double LMDBuffer[];      // q-  -
double HLMBuffer[];      //  q-     
double EMA_HLMBuffer[];  // r- 1- EMA
double DEMA_HLMBuffer[]; // s- 2- EMA
//---  
int    begin1, begin2, begin3, begin4; //  ,     
int    rates_total_min; //     
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---        
   //   ;     
   //   #0
   SetIndexBuffer(0,MainBuffer,INDICATOR_DATA);             // u- 3- EMA
   //     ;    
   SetIndexBuffer(1,HMUBuffer,INDICATOR_CALCULATIONS);      // q-  -
   SetIndexBuffer(2,LMDBuffer,INDICATOR_CALCULATIONS);      // q-  -
   SetIndexBuffer(3,HLMBuffer,INDICATOR_CALCULATIONS);      //  q-     
   SetIndexBuffer(4,EMA_HLMBuffer,INDICATOR_CALCULATIONS);  // r- 1- EMA
   SetIndexBuffer(5,DEMA_HLMBuffer,INDICATOR_CALCULATIONS); // s- 2- EMA
/*
//---   #0 (Main)
   PlotIndexSetString(0,PLOT_LABEL,"HLM");             //     #0
   PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_LINE);    //  : DRAW_LINE - 
   PlotIndexSetInteger(0,PLOT_LINE_COLOR,Blue);        //    : Blue - 
   PlotIndexSetInteger(0,PLOT_LINE_STYLE,STYLE_SOLID); //  : STYLE_SOLID -  
   PlotIndexSetInteger(0,PLOT_LINE_WIDTH,1);           //  
*/
//---    
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---
   begin1=q-1;        //                             -  : HLMBuffer[], HMUBuffer[], LMDBuffer[]
   begin2=begin1+r-1; // or =(q-1)+(r-1)             -  : EMA_HLMBuffer[]
   begin3=begin2+s-1; // or =(q-1)+(r-1)+(s-1)       -  : DEMA_HLMBuffer[]
   begin4=begin3+u-1; // or =(q-1)+(r-1)+(s-1)+(u-1) -  : MainBuffer[]
   //
   rates_total_min=begin4+1; //     
//---        #0
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,begin4);
//---   
   string shortname=string(q)+","+string(r)+","+string(s)+","+string(u);
   IndicatorSetString(INDICATOR_SHORTNAME,"Blau_HLM("+shortname+")");
//--- OnInit done
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,     //   
                const int prev_calculated, //     
                const datetime &Time[],    // Time
                const double &Open[],      // Open
                const double &High[],      // High
                const double &Low[],       // Low
                const double &Close[],     // Close
                const long &TickVolume[],  // Tick Volume
                const long &Volume[],      // Real Volume
                const int &Spread[]        // Spread
               )
  {
   int i,pos;
//---      
   if(rates_total<rates_total_min) return(0);
//---   q-     
   //   (pos),   /   q-     
   //     HLMBuffer[], HMUBuffer[], LMDBuffer[]
   if(prev_calculated==0)      //   
     {
      pos=begin1;              //  c  ,    
      for(i=0;i<pos;i++)       //   
        {
         HLMBuffer[i]=0.0;     //  
         HMUBuffer[i]=0.0;     //
         LMDBuffer[i]=0.0;     //
        }
     }
   else pos=prev_calculated-1; //     
   //     HLMBuffer[], HMUBuffer[], LMDBuffer[]
   for(i=pos;i<rates_total;i++)
     {
      HMUBuffer[i]=High[i]-High[i-(q-1)];    HMUBuffer[i]=(HMUBuffer[i]>0)?HMUBuffer[i]:0;
      LMDBuffer[i]=-1*(Low[i]-Low[i-(q-1)]); LMDBuffer[i]=(LMDBuffer[i]>0)?LMDBuffer[i]:0;
      HLMBuffer[i]=HMUBuffer[i]-LMDBuffer[i];
     }
//---   EMA
   // r- 1- EMA
   ExponentialMAOnBufferWB(
                           rates_total,     //   
                           prev_calculated, //     
                           begin1,          //         
                           r,               //  
                           HLMBuffer,       //  
                           EMA_HLMBuffer    //  
                          );
   // s- 2- EMA
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin2,s,EMA_HLMBuffer,DEMA_HLMBuffer);
   // u- 3- EMA (   #0)
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin3,u,DEMA_HLMBuffer,MainBuffer);
//--- OnCalculate done. Return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
